;;; maven-central.el --- Maven central auxility

;;; Commentary:
;;

(require 'json)
(require 'url-handlers)
(require 'maven)

;;; Code:

(defvar maven-central.buffer-name "*Maven Central*")

(defconst maven-central.search-url "http://search.maven.org/solrsearch/select?wt=json&rows=20")

(defun maven-central.versions-url (groupId artifactId)
  (concat maven-central.search-url "&core=gav&q=g:" (url-hexify-string groupId) (url-hexify-string " AND ") "a:" (url-hexify-string artifactId)))
;; (maven-central.versions-url "junit" "junit")

(defun maven-central.versions-callback (status)
  (let ( (buffer (current-buffer)) json )
    (with-temp-buffer
      (url-insert buffer)
      (goto-char (point-min))
      (setq json (json-read))
      ;; (switch-to-buffer maven-central.buffer-name)
      ;; (pp json)
      (message "Available versions: %s" (mapconcat (lambda (json) (cdr (assq 'v json))) (cdr (assq 'docs (cdr (assoc 'response json)))) ", "))
      )))

(defun maven-central.versions (groupId artifactId)
  "Retrieve available versions."
  (url-retrieve (maven-central.versions-url groupId artifactId) #'maven-central.versions-callback))

(defun maven-central.last-version-url (groupId artifactId)
  (concat maven-central.search-url "&q=g:" (url-hexify-string groupId) (url-hexify-string " AND ") "a:" (url-hexify-string artifactId)))
;; (maven-central.versions-url "junit" "junit")

(defun maven-central.last-version-callback (status)
  (let ( (buffer (current-buffer)) json )
    (with-temp-buffer
      (url-insert buffer)
      (goto-char (point-min))
      (setq json (json-read))
      ;; (switch-to-buffer maven-central.buffer-name)
      ;; (pp json)
      (message "Latest version: %s" (cdr (assq 'latestVersion (elt (cdr (assq 'docs (cdr (assoc 'response json)))) 0))))
      )))

(defun maven-central.last-version (groupId artifactId)
  "Retrieve last package version."
  (url-retrieve (maven-central.last-version-url groupId artifactId) #'maven-central.last-version-callback))

;;;###autoload
(defun maven-central.last-version-from-pom ()
  (interactive)
  (let (dependency groupId artifactId)
    (setq dependency (maven.parse-pom-dependency))
    (setq groupId (elt dependency 0))
    (setq artifactId (elt dependency 1))
    (when (and groupId artifactId)
      (maven-central.last-version groupId artifactId))
    ))

;;;###autoload
(defun maven-central.versions-from-pom ()
  (interactive)
  (let (dependency groupId artifactId)
    (setq dependency (maven.parse-pom-dependency))
    (setq groupId (elt dependency 0))
    (setq artifactId (elt dependency 1))
    (when (and groupId artifactId)
      (maven-central.versions groupId artifactId))
    ))

;; (maven-central.last-version "junit" "junit")

(provide 'maven-central)

;;; maven-central.el ends here
